@@ -84,6 +84,7 @@
 //!   outputting up to `max_results`rows.
 //! * **IDB.ISCAN**: `IDB.ISCAN table primary_lang fallback_lang num_skip max_results path1 path2 path3`
 //!   Again, behaves just like `IDB.SCAN` but provides i18n lookup for the given languages.
+//! * **IDB.LEN**: `IDB.LEN` reports the size of the given table.
 //! * **IDB.SHOW_TABLES**: `IDB.SHOW_TABLES` reports all tables and their usage statistics.
 //! * **IDB.SHOW_SETS**: `IDB.SHOW_SETS` reports all sets and their usage statistics.
 //! * **IDB.CONTAINS**: `IDB.CONTAINS set key1 key2 key3` reports if the given keys are contained
@@ -91,7 +92,8 @@
 //!   reported.
 //! * **IDB.INDEX_OF**: `IDB.INDEX_OF set key1 key2 key3` reports the insertion index for each
 //!   of the given keys using one-based indices.
-//!    
+//! * **IDB.CARDINALITY**: `IDB.CARDINALITY set` reports the size of the given set.
+//!
 //! # Example
 //!
 //! Imagine we have the following super simplified dataset representing some countries:
@@ -205,6 +207,8 @@ enum Commands {
     ShowSets,
     Contains,
     IndexOf,
+    Len,
+    Cardinality,
 }
 
 /// Installs an actor which handles the commands as described above.
@@ -247,7 +251,17 @@ pub fn install(platform: Arc<Platform>) {
             cmd_queue.clone(),
             Commands::Contains as usize,
         );
-        commands.register_command("IDB.INDEX_OF", cmd_queue, Commands::IndexOf as usize);
+        commands.register_command(
+            "IDB.INDEX_OF",
+            cmd_queue.clone(),
+            Commands::IndexOf as usize,
+        );
+        commands.register_command(
+            "IDB.CARDINALITY",
+            cmd_queue.clone(),
+            Commands::Cardinality as usize,
+        );
+        commands.register_command("IDB.LEN", cmd_queue, Commands::Len as usize);
     }
 }
 
@@ -285,7 +299,9 @@ async fn handle_call(
     match command {
         Some(Commands::ShowTables) => show_tables_command(&mut call, tables).complete(call),
         Some(Commands::ShowSets) => show_sets_command(&mut call, sets).complete(call),
-        Some(Commands::Contains) | Some(Commands::IndexOf) => handle_set_call(call, sets).await,
+        Some(Commands::Contains) | Some(Commands::IndexOf) | Some(Commands::Cardinality) => {
+            handle_set_call(call, sets).await
+        }
         _ => handle_table_call(call, tables).await,
     }
 }
@@ -433,6 +449,7 @@ async fn handle_table_call(mut call: Call, database: &HashMap<String, Arc<Table>
             }
             Some(Commands::Scan) => execute_scan(&mut call, table, false).complete(call),
             Some(Commands::IScan) => execute_scan(&mut call, table, true).complete(call),
+            Some(Commands::Len) => execute_table_len(&mut call, table).complete(call),
             _ => call.complete(Err(CommandError::ServerError(anyhow::anyhow!(
                 "Unknown token received: {}!",
                 token
@@ -604,7 +621,12 @@ where
         for row in results {
             response.array(request.parameter_count() as i32 - parameter_index as i32)?;
             for query in &queries {
-                emit_element(query.execute(row), response, &i18n)?;
+                emit_element(
+                    query.execute(row),
+                    response,
+                    &i18n,
+                    query.is_root_node_query(),
+                )?;
             }
         }
     }
@@ -616,6 +638,7 @@ fn emit_element(
     element: Element,
     response: &mut Response,
     i18n: &I18nContext,
+    suppress_i18n: bool,
 ) -> anyhow::Result<()> {
     if let Some(string) = element.as_str() {
         response.bulk(string)?;
@@ -626,10 +649,10 @@ fn emit_element(
     } else if element.is_list() {
         response.array(element.len() as i32)?;
         for child in element.iter() {
-            emit_element(child, response, i18n)?;
+            emit_element(child, response, i18n, false)?;
         }
     } else if element.is_object() {
-        if i18n.primary_lang.is_some() {
+        if !suppress_i18n && i18n.primary_lang.is_some() {
             if !emit_translated(element, i18n.primary_lang.as_ref(), response, i18n)?
                 && !emit_translated(element, i18n.fallback_lang.as_ref(), response, i18n)?
                 && !emit_translated(element, Some(i18n.default_lang), response, i18n)?
@@ -655,7 +678,7 @@ fn emit_translated(
     if let Some(lang) = lang {
         let translated = lang.execute(element);
         if !translated.is_empty() {
-            emit_element(translated, response, i18n)?;
+            emit_element(translated, response, i18n, false)?;
             return Ok(true);
         }
     }
@@ -691,6 +714,12 @@ fn hash_to_json(element: Element, i18n: &I18nContext) -> serde_json::value::Valu
     serde_json::json!(hash)
 }
 
+fn execute_table_len(call: &mut Call, table: Arc<Table>) -> CommandResult {
+    call.response.number(table.len() as i64)?;
+
+    Ok(())
+}
+
 /// For a set related call, we perform the lookup in the set dictionary while having
 /// exclusive access to the underlying hash map. We then pass the **Arc** reference into
 /// a separate thread so that multiple queries can be executed simultaneously.
@@ -719,6 +748,7 @@ async fn handle_set_call(mut call: Call, database: &HashMap<String, (Arc<Set>, S
         let token = call.token;
         match Commands::from_usize(token) {
             Some(Commands::Contains) => set_contains_command(&mut call, set).complete(call),
+            Some(Commands::Cardinality) => set_cardinality_command(&mut call, set).complete(call),
             Some(Commands::IndexOf) => set_index_of_command(&mut call, set).complete(call),
             _ => call.complete(Err(CommandError::ServerError(anyhow::anyhow!(
                 "Unknown token received: {}!",
@@ -760,6 +790,12 @@ fn set_index_of_command(call: &mut Call, set: Arc<Set>) -> CommandResult {
     Ok(())
 }
 
+fn set_cardinality_command(call: &mut Call, set: Arc<Set>) -> CommandResult {
+    call.response.number(set.len() as i64)?;
+
+    Ok(())
+}
+
 /// Handles administrative commands while having exclusive access to the data structures
 /// of the main actor.
 fn handle_admin(
@@ -893,6 +929,12 @@ name: Test
             .unwrap();
             assert_eq!(result[0][0], "de");
 
+            let result =
+                query_redis_async(|con| redis::cmd("IDB.LEN").arg("countries").query::<i32>(con))
+                    .await
+                    .unwrap();
+            assert_eq!(result, 3);
+
             let result = query_redis_async(|con| {
                 redis::cmd("IDB.LOOKUP")
                     .arg("countries")
@@ -1016,6 +1058,21 @@ name: Test
             assert_eq!(result[0][0].0, "Test");
             assert_eq!(result[0][0].1, "Österreich");
 
+            // Ensure "." works even when having i18n turned on...
+            let result = query_redis_async(|con| {
+                redis::cmd("IDB.ILOOKUP")
+                    .arg("countries")
+                    .arg("de")
+                    .arg("de")
+                    .arg("code")
+                    .arg("A")
+                    .arg(".")
+                    .query::<Vec<Vec<String>>>(con)
+            })
+            .await
+            .unwrap();
+            assert_eq!(result[0][0], "{\"code\":\"A\",\"iso\":{\"three\":\"aut\",\"two\":\"at\"},\"name\":{\"de\":\"Österreich\",\"en\":\"Austria\",\"xx\":\"Test\"}}");
+
             // Ensure searches work....
             let result = query_redis_async(|con| {
                 redis::cmd("IDB.SEARCH")
@@ -1148,6 +1205,15 @@ name: Test
                 0
             );
 
+            assert_eq!(
+                query_redis_async(|con| redis::cmd("IDB.CARDINALITY")
+                    .arg("test_set")
+                    .query::<i32>(con))
+                .await
+                .unwrap(),
+                3
+            );
+
             database
                 .perform(DatabaseCommand::DropSets("test".to_string()))
                 .await
